
#include "fit.h"

//
// This will copy a 2-dimensional matrix a to a 2-dimensional
// matrix s of the same size, and return the s-matrix.  Both
// will have elements of doubles.
//
// October 21, 2014
//

dmat copy(dmat a)
{
  int j,k;
  int n = a.size();
  dmat s(n, dvec(n));

  for (j=0;j<n;j++)
    for (k=0;k<n;k++)
      s[j][k] = a[j][k];

  return s;
}

//
// This will copy a vector b to a vector s of the same size
// and return the s-vector.  Both will have elements of
// doubles.
//
// October 21, 2014
//

dvec copy(dvec b)
{
  int j;
  int n = b.size();
  dvec s(n);

  for (j=0;j<n;j++)
    s[j] = b[j];

  return s;
}

//
// This will copy an svec s to the svec t.
//
// October 31, 2014
//

svec copy(svec s)
{
  int j;
  int n = s.size();
  svec t(n);
  for (j=0;j<n;j++)
    t[j] = s[j];

  return t;
}

//
// This will copy a vector b to a one-column matrix s and
// return the s-matrix.  The matrix s will have the number of
// rows equal to the number of elements of b. Note m must be 1.
//
// October 21, 2014
//
dmat copy(dvec b, int m)
{
  int j;
  int n = b.size();
  assert(m == 1);
  dmat s(n, dvec(m));

  for (j=0;j<n;j++)
    s[j][0] = b[j];

  return s;
}

//
// this will copy an n-row, 1-col matrix a to an n-element
// vector b.  Note m must be 1.
//
// October 31, 2014
//
dvec copy(dmat a, int m)
{
  int j;
  int n = a.size();                                  // number of rows in a
  assert(m == 1);
  dvec b(n);
  for (j=0;j<n;j++)
    b[j] = a[j][0];

  return b;
}
