//
// the eps_test is passed when each db[j] is such that
//
//   |db*[j]| < eps(tau + |b*[j]|) for each j
//
// where b*[j] and db*[j] are the scaled values of the parameter
// vector components and the corrections to them, respectively.
// Scaled components are used because they are dimensionless, and
// for each vector, the individual components are likely to be of
// comparable magnitude.  Note that unscaled components are
// input.  They are scaled in the function using the scale
// factors sf[j].
//
// Marquardt suggests an eps of 1.0e-5 and a tau of 1.0e-3.
// previously we chose eps = 2.0e-4 and tau = 1.0.  We'll do
// some testing of these to see which works better.
//
// This is the normal convergence test, invoked when phi
// decreases with an iteration.  If the test fails, try
// decreasing lambda by a factor of 10, and iterate again.
// October 31, 2014

#include "fit.h"

cdbl eps = 1.0e-5;
cdbl tau = 1.0e-3;

fail_pass eps_test(dvec b, dvec db, dvec sf, int npar)
{
  int j;
  fail_pass k = fail;

  for (j=0;j<npar;j++)
    if (fabs(db[j]*sf[j]) < eps*(tau + fabs(b[j]*sf[j])))
      k = pass;

  return k;
}

