#include <cassert>
#include <cmath>
#include <cstdio>                                           // for rename()
#include <cstdlib>                                          // for system()
#include <ctime>                                          // for strftime()
#include <fstream>
#include <getopt.h>
#include <iomanip>
#include <iostream>
#include <libgen.h>                                       // for basename()
#include <limits>                     // for xmax, xmin, gammq() and gser()
#include <sstream>
#include <vector>

using namespace std;

// class result;
// class opts;
// class misc;

typedef  const int            cint;
typedef  unsigned              uns;
typedef  double                dbl;
typedef  const double         cdbl;
typedef  const string      cstring;
typedef  istringstream    isstream;
typedef  ostringstream    osstream;
typedef  vector<dbl>          dvec;
typedef  vector<string>       svec;
typedef  vector<vector<dbl> > dmat;
typedef  vector<int>          ivec;

enum no_yes {no, yes};
enum fail_pass {fail, pass};                        // for convergence test

struct misc {
    int maxiter;
    int npar;                                       // number of parameters
    int npts;                                      // number of data points
    int ssize;           // size of s-vector, zero if no sigmas in datafile
    svec name;                                       // names of parameters
    dbl lambda;                                      // marquardt parameter
    string datafile;                                    // name of datafile
    dvec b;                                  // for saving parameter vector
};
struct opts {
    int sigmas;              // yes if independent sigmas supplied in col 3
    no_yes verbose;           // if verbose, shows gamma, lambda, scaled db
};

template<class T>
inline void SWAP(T &a, T &b)                    // used in gaussj() routine
        {T dum=a; a=b; b=dum;}

// prototypes
//
void a_x_b(dmat& ab, dmat a, dmat b, int n);            // product of a x b
void calculate(dvec x, dvec y, dvec s, dvec& b, misc m, opts o);
dmat copy(dmat a);                        // returns a copy of the a-matrix
dvec copy(dvec b);                        // returns a copy of the b-vector
svec copy(svec s);                 // returns a copy of the string-vector s
dmat copy(dvec b, int m);  // returns one-col matrix with elems of b-vector
dvec copy(dmat a, int m);      // returns vector with col-elems of a-matrix
fail_pass eps_test(dvec b, dvec db, dvec sf, int npar);
void err(cstring);
void warn(cstring);
void err(cstring, int);
void err(cstring, cstring);
void err(cstring, int, cstring);
void err(cstring, cstring, cstring);
dbl func(dbl x, dvec b, dvec& p, string& fs, misc& m);
dbl gamma(dvec db, dvec g, int npar);
void gaussj(dmat& a, dmat& g);
void get_data(ifstream& ifs,dvec& x,dvec& y,dvec& s,dvec& b,opts o,misc& m);
void get_g_a(dvec& g, dmat& a, dvec x, dvec y, dvec s, dvec b, misc m);
void get_opts(int, char**, opts&);
string get_string(cstring, cstring, uns);
void gopen(ifstream& ifs, cstring fname);
void gopen(ofstream& ofs, cstring fname);
dbl get_phi(dvec x, dvec y, dvec s, dvec b, misc m);
void gstrftime(string&, cstring, const struct tm*);
no_yes query_user();              // preceded by a question emitted by cout
void scale(dvec& g, dmat& a, dvec& sf, int npar);
void show(dvec g, string s);
void show(svec s, string ss);
void show(dmat a, string s);
void trimspace(string&);
void gsystem(cstring cmd);
void prn_usage(char**);
void prn_info(char**);
void mk_graph(misc m, opts o);
dbl factln(cint);
dbl gammln(cdbl);
dbl gammq(cdbl, cdbl);
dbl gser(cdbl, cdbl);
dbl gcf(cdbl, cdbl);
cdbl pi();
dbl norm(uns);
dbl stud(dbl, uns);
dbl student(uns, cdbl);
