//
// This gets x values (col 1) and y values (col 2) and
// (optional) sigmas (col 3).
// if col 3, there must be one for each data point value.
// if no col 3, make s[k] = 1.0 for equal weights
// Also gets the initial parameter values, of which there may
// be many.
//

#include "fit.h"

extern int optind;

void get_data(ifstream& ifs,dvec& x,dvec& y,dvec& s,dvec& b,opts o,misc& m)
{
  int k;
  int n;                                       // for counting data columns
  int j;                                         // for counting parameters
  dbl z;                                    // will contain x or y or sigma
  cstring ws=" \t";                                     // whitespace chars
  string line;
  isstream iss;

  for (k = 1; getline(ifs, line); ++k) {
    trimspace(line);
    n = 0;                                     // for counting data columns
    j = 0;                                       // for counting parameters
    iss.clear();                                      // clear state of iss
    iss.str(line);                                   // put the line in iss
    if (line.find("param") != string::npos) {  // collect init param values
      line = line.substr(line.find_first_of(ws));
      iss.clear();
      iss.str(line);
      while (iss >> z) {
        ++j;
        b.push_back(z);
      }
    }
    while (iss >> z) {               // while a double (or int) is detected
      ++n;                                // increment over numbers in line
      if (n==1) x.push_back(z);             // grow the x (x-values) vector
      else if (n==2) y.push_back(z);        // grow the y (y-values) vector
      else if (n==3) s.push_back(z);          // grow the s (sigmas) vector
      else err("too many entries in datafile at line no. ", k);
    }
    if ((s.size() != 0) && (s.size() != x.size())) {
      err("error in datafile near line no. ", k);
    }
  }
  m.ssize = s.size();                        // save for use in calculate()
  if (!s.size() && o.sigmas)
    err("  You've specified the \"--sigmas\" option, but\n",
    "     I don't see any sigmas in your datafile!");

  if (!s.size()) {       // if no sigs, make s[k] = 1.0 (for equal weights)
    for (k=0;k<x.size();k++) s.push_back(1.0);
  }
}
