//
// This creates datafile.py, used to create a graph
// November 11, 2014
//
#include "fit.h"

void mk_graph(misc m, opts o)
{

  string fname = m.datafile;
  int npar = m.npar;
  int npts = m.npts;
  int ssize = m.ssize;
  int k;
  dvec x;
  dvec y;
  dvec s;
  dvec b;
  dvec p;
  string fs;
  string ss;
  ifstream ifs;
  ofstream ofs;

  time_t now = time(NULL);              // get the time/date string into td
  struct tm* info = localtime(&now);
  cstring fmt = "%I:%M %p on %b %d, %Y";
  string td;
  gstrftime(td, fmt, info);

  gopen(ifs, fname);                               // open datafile (fname)
  assert(ifs);
  get_data(ifs,x,y,s,b,o,m);                       // get data into x,y,[s]
  string tfn = fname + ".tmp";                  // tfn: temporary file name

  gopen(ofs, tfn);                 // open tfn; python script will read tfn
  assert(ofs);
  for (k=0;k < npts;++k) {                // tfn will contain just the data
    ofs << x[k] << "\t" << y[k];
    if (ssize) {
      ofs << "\t" << s[k] << "\n";
    }
    else {
      ofs << "\n";
    }
  }
  ofs.close();
  b = m.b;                     // saved from final iteration in calculate()
  func(x[0],b,p,fs,m);                                  // provide b and fs
  cout << "  Here's the function string, needed to make the graph.\n"
       << "  Check its format to make sure it's correct.  Do the\n"
       << "  parentheses match?\n\n"
       << "  fs = " << fs << "\n";

  dbl xmin;
  dbl xmax;
  dbl xmin_old;
  dbl xmax_old;
  int xlim1;
  int xlim2;
  xmin = numeric_limits<dbl>::max();                      // largest double
  xmax = numeric_limits<dbl>::min();                     // smallest double
  for (k = 0;k < npts;k++) {      // determine xmin & xmax from data points
    xmin = (x[k] <= xmin) ? x[k] : xmin;
    xmax = (x[k] >= xmax) ? x[k] : xmax;
  }

  xmin_old = xmin;                             // save these for future use
  xmax_old = xmax;
  xmin = 1.05*xmin_old - 0.05*xmax_old;   // for extent of f(x) beyond data
  xmax = 1.05*xmax_old - 0.05*xmin_old;   // for extent of f(x) beyond data
  xlim1 = (1.1*xmin_old - 0.1*xmax_old) + 0.5;    // for extending x-axis
  xlim2 = (1.1*xmax_old - 0.1*xmin_old) + 0.5;    // for extending x-axis

  string pyfn = fname + ".py";     // pyfn: python filename (python script)
  string cmd = "python " + pyfn;                // command to display graph

  string title;
  string xlabel;
  string ylabel;
  string question;

  question = "  OK, Please provide a brief title for\n"
             "  your graph (see fit --help for hints).\n"
             "  (Insert \" td\" to get a \"time/data\" string.)\n"
             "  (Just hit \'return\' for no title at all.)";
  title = get_string(question,td, 2);
  cout << "  title = " << title << "\n\n";      // display the title string

  question = "  Provide also a brief x-axis label:\n"
             "  (Again hit \'return\' for no label.)";
  xlabel = get_string(question,td, 2);
  cout << "  xlabel = " << xlabel << "\n\n";   // display the xlabel string

  question = "  And also a brief y-axis label:\n"
             "  (And again hit \'return\' for no label.)";
  ylabel = get_string(question,td, 2);
  cout << "  ylabel = " << ylabel << "\n\n";   // display the ylabel string

  gopen(ofs, pyfn);                               // put script in fname.py
  ofs.clear();
  assert(ofs);
  ofs << "# A graph may be created using the command \'python file.py\'\n"
      << "# where \'file.py\' is this file.  python, numpy and matplotlib\n"
      << "# must be present on your system for this to work.  You currently\n"
      << "# have installed the versions shown below.\n"
      << "#\n"
      << "from platform import python_version\n"
      << "from pylab import *\n"
      << "print \"   python version:    \",python_version()\n"
      << "print \"   numpy version:     \",np.__version__\n"
      << "print \"   matplotlib version:\",matplotlib.__version__\n\n"
      << "# The next lines define f(x), grabbed (as fs) from func()\n"
      << "def f(x):\n"
      << "   return " << fs << "\n"
      << "# The next line determines the extent of f(x) beyond the\n"
      << "# plotting of the data.\n"
      << "x = linspace(" << xmin << ", " << xmax << ", 600)\n"
      << "plot(x,f(x), \"b\",lw=0.8)\n"
      << "# The next line loads the data from your (temporary) datafile.\n"
      << "data=np.loadtxt(\'" << tfn << "\')\n\n"     // load data from tfn
      << "# In the next line (which plots the data points), \'gd\' means\n"
      << "# green diamonds.  Other colors or point shapes may be substituted.\n"
      << "# ms=4 sets the \"markersize\" of the plotted data points to 4 pixels.\n"
      << "# If the --sigmas option is specified, the points will have error bars.\n";
  if (o.sigmas && ssize) {
    ofs << "errorbar(data[:,0], data[:,1], data[:,2], fmt='gd', ms=4, capsize=4)\n\n";
  }
  else {
    ofs << "plot(data[:,0], data[:,1], \'gd\', ms=4)\n\n";
  }
  ofs << "# This next line extends the axes to provide needed white space.\n"
      << "xlim(" << xlim1 << ", " << xlim2 << ")\n\n"
      << "# The next three lines of code set your title and labels for the axes.\n"
      << "# You can insert newline characters (\'\\n\') in your string if needed,\n"
      << "# and/or incorporate \"TeX\" characters by enclosing them in \'$\' signs.\n"
      << "title(" << title << ")\n"
      << "xlabel(" << xlabel << ")\n"
      << "ylabel(" << ylabel << ")\n\n"
      << "# Additional lines of text may be placed anywhere on your graph by\n"
      << "# editing this file after the program ends.  Here's a template:\n"
      << "# text(X, Y, r'$T^2 =\\alpha M + \\beta$, is plotted', fontsize=13)\n"
      << "# Here (X,Y) is the lower left position of the text string in user units,\n"
      << "# and characters between the \'$\' signs are \"TeX\" math characters.\n"
      << "# The initial \'r\' will be needed if you insert \"TeX\" characters.\n\n"
      << "# Now we put a grid on the plot, and issue the \"show()\" command to\n"
      << "# display the plot on your screen.  (tight-layout() provides space for\n"
      << "# multi-line titles and labels.)\n"
      << "tight_layout()\n"
      << "grid()\n"
      << "show()\n";
  ofs.close();
  gsystem(cmd);

  question = "\n  Would you like to remove \"" + tfn + "\"?  (Say no\n"
             "  if you want to edit the python script.) [yes],no";
  string ans = get_string(question, td, 0);
  if (ans[0] == '\0' || ans[0] == 'y' || ans[0] == 'Y') {
    remove(tfn.c_str()); // return key alone returns ans[0] = '\0'
  }
  cout << "\n";
}

//
// This next function asks a question,
// then gets and returns an answer.
// If j=2, and the string submitted contains " td",
// a time/date string will exist in the returned string,
// and the returned string will be enclosed in single quotes.
// See above for examples.
//
string get_string(cstring question, cstring td, uns j)
{
  string line;
  if (j) cerr << question << "\n";
  else cerr << question << " ";
  getline(cin, line);
  if (j==2) {
    if (line.find(" td") != string::npos) {
      line.replace(line.find("td"),2,td);
    }
    line = "\'" + line + "\'";    // enclose by "'" chars, needed by python
  }
  return line;
}

