//
// prints a help message on the screen
// fot the generic fit program
// November 26, 2014
//
#include "config.h"                                // for PACKAGE_BUGREPORT
#include "fit.h"

void prn_info(char** argv)
{
  string pgname = basename(argv[0]);                 // chops long pathname

  if (pgname.rfind(".exe") != string::npos) {
    pgname.replace(pgname.find(".exe"),4,"");
  }

  cout << "\n"
       << "  This is the generic \033[1mhelp\033[0m file that is applicable to every \033[1mfit\033[0m program.\n\n"

       << "  \033[1mfit\033[0m will fit a specified function \033[1mf(x, b[0], b[1], ...)\033[0m to a set of data\n"
       << "  points \033[1m(x[i], y[i])\033[0m, by adjusting the set of parameters \033[1mb[k]\033[0m so as to\n"
       << "  minimize the sum of the squares of the quantity \033[1m(f(x[i], b[0],...)-y[i])\033[0m.\n"
       << "  Such a process is commonly called a \"least-squares\" fit.\n\n"

       << "  \033[1mHOW TO PREPARE A DATAFILE:\033[0m\n\n"
       << "  The values of the data points \033[1m(x[i], y[i])\033[0m must be contained in the datafile\n"
       << "  that is the argument of the fit program.  This datafile must have either two\n"
       << "  or three columns of numbers.  The first column must contain the \033[1mx[i]\033[0m values,\n"
       << "  and the second column the \033[1my[i]\033[0m values.  The (optional) third column, if it\n"
       << "  exists, must contain the uncertainties in the values of \033[1my[i]\033[0m, denoted in the\n"
       << "  program by \033[1ms[i]\033[0m.  These uncertainties may be independently estimated, say by\n"
       << "  measuring a number of the \033[1my[i]\033[0m for a given \033[1mx[i]\033[0m and finding the standard\n"
       << "  deviation of the mean, or, if the \033[1my[i]\033[0m happen to be drawn from a Poisson\n"
       << "  distribution (such as the pulses from a Geiger counter), as the square root\n"
       << "  of \033[1my[i]\033[0m.  Such independent uncertainties are also known as \"standard errors\".\n\n"

       << "  If the \033[1ms[i]\033[0m are independently determined, the fit program should be run with\n"
       << "  the \"\033[1m--sigmas\033[0m\" option, in which case the value of chi-square will be\n"
       << "  available for testing goodness of fit, and error bars will be displayed on\n"
       << "  the resulting graph.\n\n"

       << "  If independent uncertainties are not available, but if the points are not\n"
       << "  to be equally weighted, relative values of the s[i], proportional to\n"
       << "  \033[1m1/sqrt(w[i])\033[0m (\033[1mw[i]\033[0m is the weight of the ith point), may be supplied.  In\n"
       << "  this case, the \"\033[1m--sigmas\033[0m\" option should not be invoked, no value of\n"
       << "  chi-square is available, and no error bars will be displayed on the graph.\n\n"

       << "  If the third column in the datafile is empty (no uncertainties are supplied),\n"
       << "  each point will be given equal weight, and here also, no chi-square is\n"
       << "  available.\n\n"

       << "  Initial guesses for the values of the parameters \033[1mb[k]\033[0m must also be supplied\n"
       << "  in the datafile, on a single line, following any initial word beginning with\n"
       << "  \"\033[1mparam\033[0m\", such as \"\033[1mparams:\033[0m\" or \"\033[1mparameters:\033[0m\". (To guess values for the \033[1mb[k]\033[0m,\n"
       << "  it may be helpful first to plot your data, either by hand, or by using our\n"
       << "  \033[1mplotdata\033[0m program.)  Thus, the first few lines of a datafile might look like\n"
       << "  this:\n\n"

       << "          parameters: 1500.0 20.0 10.0 600.0\n"
       << "        -----------------------------------------\n"
       << "          data for a single gaussian lineshape from\n"
       << "              the X-ray fluorescence experiment\n"
       << "\n"
       << "          CHAN#  COUNTS  SIGMA\n"
       << "        -----------------------------------------\n"
       << "            9     490    22.14\n"
       << "           10     517    22.74\n"
       << "           11     530    23.02\n"
       << "           12     567    23.81\n\n"

       << "  Note from the above example that in general, blocks of text and blank lines\n"
       << "  in a datafile will be ignored.  Moreover, if ANY line in a datafile is\n"
       << "  preceded by a \"sharp\" character (\033[1m#\033[0m) it will also be ignored, so individual\n"
       << "  data points may eliminated in this way if desired.\n\n"

       << "  \033[1mHOW TO SPECIFY THE FORM OF THE FUNCTION TO BE FIT TO THE DATA:\033[0m\n\n"

       << "  The user must supply a file, called \"\033[1mfunc.cc\033[0m\", that contains the source code\n"
       << "  describing the exact form of the function to be fit to the data.  This file\n"
       << "  must then be compiled, using a C++ compiler, so as to incorporate the\n"
       << "  existing library of functions that comprise the rest of the fit program.\n\n"

       << "  A template example of \"\033[1mfunc.cc\033[0m\", called \"\033[1mfunc.cc.gauss\033[0m\", is present in the\n"
       << "  \"\033[1msrc\033[0m\" directory.  This example, which is simply a copy of the \"\033[1mfunc.cc\033[0m\"\n"
       << "  that also appears in the \"\033[1msrc\033[0m\" directory, is suitable for fitting the data\n"
       << "  contained in the sample datafile \"\033[1mtestfiles/gauss\033[0m\", described above.\n"
       << "  Thus, compiling the fit program initially should produce a working binary\n"
       << "  that will work to fit the data in \"\033[1mtestfiles/gauss\033[0m\" to a single Gausian\n"
       << "  lineshape, using the command \033[1mfit --sigmas testfiles/gauss\033[0m.\n\n"

       << "  For fitting data to other hypothesized functions, \"\033[1mfunc.cc\033[0m\" must be modified\n"
       << "  accordingly.  (NOTE: Once your initial compilation and installation of this\n"
       << "  fit program---which installs the library of generic fit functions---you may\n"
       << "  use that library to compile new programs to fit data to other hypothesized\n"
       << "  functions.  See \033[1mhow_to_compile_fit_for_a_new_func.pdf\033[0m in the \033[1mdoc\033[0m directory.)\n\n"

       << "  There are four sections of code in \"\033[1mfunc.cc\033[0m\" that must be changed.  Each is\n"
       << "  marked with dollar signs.  The first, usually a single line of code,\n"
       << "  produces the value of the function \033[1mf(x,b)\033[0m for a single value of \033[1mx\033[0m and the\n"
       << "  current set of parameters \033[1mb[k]\033[0m that are needed by the program.  The second\n"
       << "  produces the text string \033[1mfs\033[0m that is needed to make the graph, via the Python\n"
       << "  script produced by the program.\n\n"

       << "  The third simply supplies the vector of parameter names, used by the program\n"
       << "  in the display of its output.  The fourth provides the components of the\n"
       << "  vector of partial derivatives \033[1mp[k] = df/db[k]\033[0m.\n\n"

       << "  The second and fourth sections are the ones most susceptible to coding\n"
       << "  errors---errors that will likely result in a non-working program.  For help\n"
       << "  in trouble-shooting such errors, look at the code in \033[1msrc/main.cc\033[0m.  In this\n"
       << "  piece of code are a number of lines, currently commented out, that if\n"
       << "  un-commented, will provide output that could be useful.\n\n"

       << "  \033[1mA FEW NOTES ABOUT USING THIS PROGRAM:\033[0m\n\n"

       << "  Once you have a correctly coded \"\033[1mfunc.cc\033[0m\", the \033[1m" << pgname << "\033[0m program may be\n"
       << "  run.  There are four possible options:\n\n"

       << "  Usage: \033[1m" << pgname << "\033[0m [--help] [--version] [--sigmas] [--verbose] \033[1mdatafile\033[0m\n"
       << "  Single letter options (-h, -v, -s, -V) are also available.\n\n"

       << "  The \033[1m--help\033[0m option will display this file and exit.  The \033[1m--version\033[0m option\n"
       << "  will display the version number and exit.  The \033[1m--sigmas\033[0m option should be\n"
       << "  invoked if the third column in your data file contains independently \n"
       << "  determined values of the standard errors in the \033[1my[i]\033[0m, as described above, \n"
       << "  and the \033[1m--verbose\033[0m option provides extra information in the output of the\n"
       << "  program.\n\n"

       << "  If you would like to save a copy of the program\'s output (which by default\n"
       << "  would appear on your monitor screen), you may redirect the output to a file\n"
       << "  with a command like \033[1m" << pgname << " datafile > outfile\033[0m.  Then the command\n"
       << "  \033[1mcat outfile\033[0m will display the \033[1moutfile\033[0m on the screen.\n\n"

       << "  Trouble or bug reports?  Send an email to Peter Scott <" << PACKAGE_BUGREPORT << ">.\n\n"

       << "                 ==========================================\n\n";

}

